;======================================================================
.segment "CODE"
;======================================================================

.proc start_metasprite
  rep #$30	;A=16, X/Y=16
  lda #ObjectTable
  tcd
  ldy #$0000
  bra continue_metasprite_finder

metasprite_finder:
  tdc
  clc
  adc #ObjectTableEntrySize
  tcd
  cmp #ObjectTableSize+ObjectTable
  bne continue_metasprite_finder
  sty a:SpriteCount
  lda #$0000
  tcd
  rts

continue_metasprite_finder:
  ldx ObjectTableSlot::MetaspriteTableOffset
  beq metasprite_finder
  lda a:$0000,x			;1st byte = sprite X position (value 0-255)
  cmp #$0001
  bne attribute_condenser
  brl single_metasprite

attribute_condenser:
  sta a:MetaspriteCount
  lda ObjectTableSlot::FlipBits
  sta a:MetaspriteAttributes

horizontal_flip_check:
  stz a:HFlipMask
  lda ObjectTableSlot::FlipBits
  bit #$4000
  beq vertical_flip_check
  lda #$FFFF
  sta a:HFlipMask

vertical_flip_check:
  stz a:VFlipMask
  lda ObjectTableSlot::FlipBits
  bit #$8000
  beq metasprite_loop
  lda #$FFFF
  sta a:VFlipMask

metasprite_loop:
  cpy #$0200			;sees if all 128 sprites are used up
  beq metasprite_done
  and #$00FF
  sta a:SpriteBuf1+1,y		;store sprite Y position in SpriteBuf1+1,y
  lda a:MetaspriteCount		;if MetaspriteCount is zero, then we're done.  Otherwise we have
  beq metasprite_done		;metasprites to iterate over and populate for DMA (see VBlank)

  lda a:$0002,x			;1st byte = sprite X position (value 0-255)
  eor a:HFlipMask
  clc
  adc ObjectTableSlot::XPosition
  cmp #256
  bcc sprite_x_not_out_of_bounds
  cmp #65504
  bcs sprite_x_not_out_of_bounds
  bra sprite_out_of_bounds

sprite_x_not_out_of_bounds:
  and #$01FF
  sta a:SpriteBuf3,y		;Store sprite X position SpriteBuf1+y
  and #$00FF
  sta a:SpriteBuf1,y		;Store sprite X position SpriteBuf1+y
  lda a:$0004,x			;2nd byte = sprite Y position (value 0-255)
  eor a:VFlipMask
  clc
  adc ObjectTableSlot::YPosition
  cmp #224
  bcc sprite_y_not_out_of_bounds
  cmp #65504
  bcs sprite_y_not_out_of_bounds
  bra sprite_out_of_bounds

sprite_y_not_out_of_bounds:
  sta a:SpriteBuf1+1,y
  lda a:$0006,x
  sta a:SpriteBuf3+2,y		;sprite size
  lda a:$0008,x
  eor a:MetaspriteAttributes
  sta a:SpriteBuf1+2,y		;palette/character word
  iny
  iny
  iny
  iny

sprite_out_of_bounds:
  txa
  clc
  adc #$0008
  tax
  dec a:MetaspriteCount		;decrement MetaspriteCount by 1
  bra metasprite_loop		;back to the loop...

metasprite_done:
  brl metasprite_finder

single_metasprite:
  cpy #$0200			;sees if all 128 sprites are used up
  beq metasprite_done
  and #$00FF
  sta a:SpriteBuf1+1,y		;store sprite Y position in SpriteBuf1+1,y

  lda a:$0002,x			;1st byte = sprite X position (value 0-255)
  clc
  adc ObjectTableSlot::XPosition
  cmp #256
  bcc single_sprite_x_not_out_of_bounds
  cmp #65504
  bcs single_sprite_x_not_out_of_bounds
  bra single_sprite_out_of_bounds

single_sprite_x_not_out_of_bounds:
  and #$01FF
  sta a:SpriteBuf3,y		;store sprite X position SpriteBuf1+y
  and #$00FF
  sta a:SpriteBuf1,y		;store sprite X position SpriteBuf1+y
  lda a:$0004,x			;2st byte = sprite Y position (value 0-255)
  clc
  adc ObjectTableSlot::YPosition
  cmp #224
  bcc single_sprite_y_not_out_of_bounds
  cmp #65504
  bcs single_sprite_y_not_out_of_bounds
  bra single_sprite_out_of_bounds

single_sprite_y_not_out_of_bounds:
  sta a:SpriteBuf1+1,y
  lda a:$0006,x
  sta a:SpriteBuf3+2,y		;Sprite size
  lda a:$0008,x
  eor ObjectTableSlot::FlipBits
  sta a:SpriteBuf1+2,y		;Palette/Character word
  iny
  iny
  iny
  iny

single_sprite_out_of_bounds:
  brl metasprite_finder
.endproc